@@ -299,6 +299,7 @@ define([
 					ucr: this._loadUCRVariables()
 				}).then(lang.hitch(this, function(results) {
 					this._ldapBase = results.metaInfo.ldap_base;
+					this._metaInfo = results.metaInfo;
 					this._reports = results.reports;
 					this._default_columns = results.columns;
 					this.renderSearchPage(results.containers, results.metaInfo);
@@ -604,6 +605,10 @@ define([
 				}
 				return text;
 			});
+			var _viewDescriptionText = lang.hitch(this, function() {
+				text = _('View the deleted object.');
+				return text;
+			});
 			var _deleteDescriptionText = lang.hitch(this, function() {
 				var text = {
 					'users/user'          : _('Delete the selected users.'),
@@ -703,7 +708,8 @@ define([
 				iconClass: 'plus',
 				isContextAction: false,
 				isStandardAction: true,
-				callback: lang.hitch(this, 'showNewObjectDialog')
+				canExecute: lang.hitch(this, '_canAdd'),
+				callback: lang.hitch(this, 'showNewObjectDialog'),
 			}, {
 				name: 'edit',
 				label: _('Edit'),
@@ -793,6 +799,44 @@ define([
 				})
 			}];
 
+			// FIXME: overwrite actions for deleted object
+			if ('recyclebin/removedobject' === this.moduleFlavor) {
+				var actions = [
+					{
+						name: 'view',
+						label: _('View'),
+						description: _viewDescriptionText(),
+						isStandardAction: true,
+						isMultiAction: false,
+						canExecute: lang.hitch(this, '_canView'),
+							callback: lang.hitch(this, function(ids, items) {
+							this.createDetailPage('view', items[0].objectType, ids[0]);
+						})
+					}, {
+						name: 'restore',
+						label: _('Restore'),
+						description: _('Restore deleted objects from recyclebin.'),
+						isStandardAction: true,
+						isMultiAction: true,
+						canExecute: lang.hitch(this, '_canRestore'),
+						callback: lang.hitch(this, function(ids, objects) {
+						        this.restoreObjects(objects);
+						})
+					}, {
+						name: 'delete',
+						label: _('Delete'),
+						description: _deleteDescriptionText(),
+						isStandardAction: true,
+						isMultiAction: true,
+						iconClass: 'trash',
+						canExecute: lang.hitch(this, '_canDelete'),
+						callback: lang.hitch(this, function(ids, objects) {
+							this.removeObjects(objects);
+						})
+					},
+				];
+			}
+
 			if ('navigation' !== this.moduleFlavor && this._reports.length) {
 				actions.push({
 					name: 'report',
@@ -829,6 +873,9 @@ define([
 					if ('navigation' == this.moduleFlavor && (this._searchForm._widgets.objectType.get('value') == '$containers$' || items[0].$childs$ === true)) {
 						return 'workaround';
 					}
+					if ('recyclebin/removedobject' == this.moduleFlavor) {
+						return 'view';
+					}
 					return 'edit';
 				})
 			});
@@ -1282,6 +1329,24 @@ define([
 			domClass.add(this._grid._grid.domNode, 'umcDynamicHeight-55');
 		},
 
+		_canAdd: function(item) {
+			// FIXME: can't check operations here, policies/policy just has search
+			// navigation does not have metainfo
+			// return this._metaInfo.operations.indexOf('add') !== -1;
+			if (this.moduleFlavor === 'recyclebin/removedobject') {
+				return false;
+			}
+			return true;
+		},
+
+		_canRestore: function(item) {
+			return item.$operations$.indexOf('restore') !== -1;
+		},
+
+		_canView: function(item) {
+			return item.$operations$.indexOf('read') !== -1;
+		},
+
 		_canEdit: function(item) {
 			return item.$operations$.indexOf('edit') !== -1;
 		},
@@ -1690,6 +1755,12 @@ define([
 					},
 					description: _('Value of the LDAP object.')
 				};
+				var orgObjectTypeColumn = {
+					name: 'originalObjectType',
+					label: _('Object type'),
+					description: _('Original objec type of the removed object.')
+					// TODO: transform users/user -> User?!
+				};
 
 				var identifies = this._searchForm ? this.identityProperty() : null;
 				var selected_value = this._searchForm ? this._searchForm._widgets.objectProperty.get('value') : 'None';
@@ -1707,6 +1778,9 @@ define([
 //				if (customColumns.length) {
 					columns.push(valueColumn);
 				}
+				if (objectType == 'recyclebin/removedobject' && selected_value !== 'originalObjectType') {
+					columns.push(orgObjectTypeColumn);
+				}
 				if (this._tree || numObjTypes > 2) {
 					columns.push(typeColumn);
 				}
@@ -1749,6 +1823,69 @@ define([
 			}));
 		},
 
+		restoreObjects: function(objects) {
+			if (!objects.length) {
+				return;
+			}
+			var ids = array.map(objects, function(object) { return object.id || object.$dn$; });
+			var _msg = lang.hitch(this, function(n) {
+				return _.ngettext(
+					'Please confirm the restore of the deleted object:',
+					'Please confirm the restore of the %d selected deleted objects:', n);
+			})
+			var msg = _msg(objects.length);
+
+
+			array.forEach(objects, function(object) {
+				msg += lang.replace('<div>{0}</div>', [this.iconFormatter(object)]);
+			}, this);
+
+			// let user confirm deletion
+			dialog.confirm(msg, [{
+				name: 'cancel',
+				'default': true,
+				label: _('Cancel')
+			}, {
+				name: 'restore',
+				label: _('Restore')
+			}]).then(lang.hitch(this, function(response) {
+				if (response != 'restore') {
+					return;
+				}
+
+				// FIXME: add to module store
+				this.moduleStore.restore = function(object, options, handleErrors) {
+					return this._genericCmd('restore', {
+						object: object,
+						options: options || null
+					}, handleErrors);
+				}
+
+				// restore objects
+				var transaction = this.moduleStore.transaction();
+				array.forEach(ids, function(id) {
+					this.moduleStore.restore(id, null)
+				}, this);
+
+				// commit and error handling
+				this.standbyDuring(transaction.commit()).then(lang.hitch(this, function(data) {
+					var message = '<p>' + _('The following object(s) could not be restored:') + '</p><ul>';
+					var success = true;
+					array.forEach(data, function(result) {
+						if (!result.success) {
+							success = false;
+							message += lang.replace('<li>{0}: {1}</li>', [entities.encode(result.$dn$), entities.encode(result.details)]);
+						}
+					}, this);
+					message += '</ul>';
+					if (!success) {
+						dialog.alert(message);
+					}
+					this.resetPathAndReloadTreeAndFilter(ids);
+				}));
+			}));
+		},
+
 		removeObjects: function(/*String|String[]*/ _ids, /*Boolean?*/ isContainer, /*Boolean?*/ cleanup, /*Boolean?*/ recursive) {
 			var _msg = lang.hitch(this, function(n) {
 				var text = {
